(function(){
    /* istanbul ignore next */
    window.app = window.app || {};
    window.app.ads = {
        adColonyFullConfig: {
            ios: {
                app_id: "app2bc12224431048cabd",
                zone_ids: [ "vz5181b4226e1d41b08e" ]
            },
            android: {
                app_id: "appeb38e78d7fae4b62b6",
                zone_ids: [ "vzea7c1fd105714fedbc" ]
            }
        },
        chartboostFullConfig: {
            android: {
                app_id: "5501be00c909a63c70ddb4c6",
                app_sig: "8d331519c195ebce4684f2fe890804f99164150f"
            },
            ios: {
                app_id: "4fdba7c7f776592d51000025",
                app_sig: "f0d85206c2cf5dc85a6881ec5d5b91a991a36a68"
            },
            CBLocationStartup: "Startup",
            CBLocationHomeScreen: "Home Screen",
            CBLocationMainMenu: "Main Menu",
            CBLocationGameScreen: "Game Screen",
            CBLocationAchievements: "Achievements",
            CBLocationQuests: "Quests",
            CBLocationPause: "Pause",
            CBLocationLevelStart: "Level Start",
            CBLocationLevelComplete: "Level Complete",
            CBLocationTurnComplete: "Turn Complete",
            CBLocationIAPStore: "IAP Store",
            CBLocationItemStore: "Item Store",
            CBLocationGameOver: "Game Over",
            CBLocationLeaderBoard: "Leaderboard",
            CBLocationSettings: "Settings",
            CBLocationQuit: "Quit",
            CBLocationDefault: "Default"
        },
        configureAdColonyAdverts: function () {
            app.ads.adColonyConfig = app.isAndroid ? app.ads.adColonyFullConfig.android : app.ads.adColonyFullConfig.ios;
            AdColony.initialize( app.ads.adColonyConfig.app_id, app.ads.adColonyConfig.zone_ids, {
                debug: false, // iOS only
                optionString: "version:1.0,store:google" // Android only
            } );
        },
        showAdColonyVideoAd: function ( success, failure ) {
            AdColony.showVideoAd( app.ads.adColonyConfig.zone_ids[ 0 ], app.ads.recordAdColonyViewSuccess(success), failure );
        },
        recordAdColonyViewSuccess: function( success ) {
            return function() {
                app.flurryAnalytics.logEvent('advertviewed', { type: 'adcolony', game: handlers.currentGame}, function() { console.log('recorded adcolony ad viewed');}, function() { console.error('couldn\'t record adcolony ad viewed');});
                success();
            };
        },
        configureChartBoost: function () {
            app.ads.chartBoostConfig = app.isAndroid ? app.ads.chartboostFullConfig.android : app.ads.chartboostFullConfig.ios;
            Chartboost.init( app.ads.chartBoostConfig.app_id, app.ads.chartBoostConfig.app_sig );
        },
        showChartboostAd: function ( success, failure, location ) {
            Chartboost.showInterstitial( app.ads.recordChartboostAdViewSuccess(success), failure, location );
        },
        recordChartboostAdViewSuccess: function( success ) {
            return function() {
                app.flurryAnalytics.logEvent('advertviewed', { type: 'chartboost', game: handlers.currentGame}, function() { console.log('recorded chartboost ad viewed');}, function() { console.error('couldn\'t record chartboost ad viewed');});
                success();
            };
        },
        advertConfiguration: {
            levelStartAttemptBased : {
                attemptNumberForFirstAdvert: 2,
                attemptFrequencyForRepeatAdverts: 2
            },
            sequence : ["video", "static"]
        },
        attemptNumber: 0,
        typeIndex: 0,
        clearAdvertAttemptNumber: function(preGame) {
            app.ads.attemptNumber = preGame ? app.ads.advertConfiguration.levelStartAttemptBased.attemptNumberForFirstAdvert - 1 : 0;
            app.ads.typeIndex = app.ads.advertConfiguration.sequence.length - 1;
        },
        incrementAdvertAttemptAndConditionallyShowAdvert: function(success) {
            app.ads.attemptNumber++;
            if (app.ads.advertConfiguration.levelStartAttemptBased) {
                app.ads.conditionallyShowAttemptBasedAdverts(success);
            } else {
                app.util.queueFunctionToRunAfterCurrentThread(success);
            }
        },
        conditionallyShowAttemptBasedAdverts: function(success) {
            var showAdvertThisTime = (app.ads.attemptNumber - app.ads.advertConfiguration.levelStartAttemptBased.attemptNumberForFirstAdvert) % app.ads.advertConfiguration.levelStartAttemptBased.attemptFrequencyForRepeatAdverts === 0;
            if (showAdvertThisTime) {
                app.ads.showAdvert(success);
            } else {
                app.util.queueFunctionToRunAfterCurrentThread(success);
            }
        },
        showAdvert: function(success) {
            app.ads.typeIndex = (app.ads.typeIndex + 1) % app.ads.advertConfiguration.sequence.length;
            switch(app.ads.advertConfiguration.sequence[app.ads.typeIndex]) {
                case "video":
                    app.ads.showVideoAdvert(success, function(videoError) {
                        console.log('Video advert failed, falling back to static advert', videoError);
                        app.ads.showStaticAdvert(success, function(staticError) {
                            console.log('Fallback static advert failed, resuming the game', staticError);
                            success();
                        });
                    });
                    break;
                case "static":
                    app.ads.showStaticAdvert(success, function(staticError) {
                        console.log('Static advert failed, falling back to video advert', staticError);
                        app.ads.showVideoAdvert(success, function(videoError) {
                            console.log('Fallback video advert failed, resuming the game', videoError);
                            success();
                        });
                    });
                    break;
                default:
                    app.util.queueFunctionToRunAfterCurrentThread(success);
                    break;
            }
        },
        showVideoAdvert: function(success, failure) {
            if (cordova.platformId === 'ios') {
                console.log('Disabled Video Adverts on iOS');
                setTimeout(failure, 0);
            } else {
                var onVideoSuccess = app.ads.adColonyCompleted(success);
                window.addEventListener('adcolony.adcompleted', onVideoSuccess);
                app.ads.showAdColonyVideoAd(function () {
                    // On iOS, the success function is called back immediately, not what we are after
                    /* istanbul ignore else */
                    if (app.isAndroid) {
                        onVideoSuccess();
                    }
                }, failure);
            }
        },
        showStaticAdvert: function(success, failure) {
            app.ads.showChartboostAd(success, failure, app.ads.chartboostFullConfig.CBLocationDefault);
        },
        adColonyCompleted: function(success) {
            var succeeded = false;
            var returnable = function() {
                // In case this function is called repeatedly, we only fire on the first success call.
                if (!succeeded) {
                    succeeded = true;
                    window.removeEventListener('adcolony.adcompleted', returnable);
                    app.ui.rotateDevice();
                    success();
                }
            };
            return returnable;
        }
    };
})();
