var CommsWrapper = new (function() {
    var hash = new PM.Hash();
    var isParent = hash.get("handle") ? false : true;
    var self = this;
    this.handle;
    if (isParent) {
        this.handle = 'mainframe';
    } else {
        var referrer = hash.get("referrer");
        var parsedReferrer = PM.parseUrl(referrer ).href;
        PM.initChildIframe(parsedReferrer);
        this.handle = 'parent';
        PM.addAuthorizedUrl(parsedReferrer);
    }
    
    var outgoingMessages = {};
    var handlers = {};
    var uid = 0;
    /**
     * This function will post a message out and store an ID of this message internally. Calling functions are given
     * a return value which has a result function. When the result function is called with a callback function, it
     * registers the callback function as something that should be run, when the remote side has performed a long
     * standing action. 
     * @param command - The command to send to the remote side
     * @param message - The message or data you want to send
     * @returns {{result: Function}}
     */
    this.postMessage = function(command, message) {
        var data = JSON.parse(JSON.stringify({
            pmid: uid++,
            command: command,
            data: message
        }));
        var returnable = {
            result: function(callbackFunction, context) {
                outgoingMessages[data.pmid] = {func: callbackFunction, context: context};
            }
        };
        PM.postMessage(this.handle, 'm', data, 'm');
        return returnable;
    };
    /**
     * You can call this function to register your handlers for certain commands. 
     * @param command
     * @param handler
     */
    this.addCommandHandler = function(command, handler) {
        handlers[command] = handler;
    };
    /**
     * This function causes an iFrame to be loaded with the ID of the handle (which is mainframe), and that contains the
     * necessary parameters to allow the iFrame to register itself as a child
     * @param url
     */
    this.loadIFrame = function(url) {
        PM.removePMIframe(document.getElementById(this.handle));
        var iFrame = PM.preloadUrl(this.handle, url);
        PM.clearAuthorizedUrls();
        PM.addAuthorizedUrl(url);
        return iFrame;
    };

    this.messageReceived = function(message) {
        message = message.data;
        if (message && typeof message.pmidanswer !== 'undefined') {
            // If we've got a message which is an answer message, then call any registered callback handlers
            var handler = outgoingMessages[message.pmidanswer];
            delete outgoingMessages[message.pmidanswer];
            if (handler && handler.func) {
                handler.func.call( handler.context, message.data );
            }
        } else if (message && message.command && handlers[message.command]) {
            message.result = function(data) {
                PM.postMessage(self.handle, 'm', {
                    pmidanswer: message.pmid,
                    data: data
                }, 'm');
            };
            handlers[message.command](message);
        }
    };
    PM.registerListener('m', this.messageReceived);
    return this;
})();