(function(){
    /* istanbul ignore next */
    window.app = window.app || {};
    window.app.download = {
        tempfileBase: Date.now(),
        tempfile: 0,
        numRetries: 3,
        /**
         * This function exists because on Android, you can't copy the files locally, even though you can serve a web-server
         * from that folder. So this sparks up a web-server, downloads the files locally, and then shuts down the web-server.
         * This function assumes that the webserver is already stopped.
         * @param filesArray
         * @param successFunction
         * @param failureFunction
         */
        downloadLocalFiles: function ( filesArray, game, successFunction, failureFunction ) {
            var numberOfFilesDownloaded = 0;
            
            var downloadFile = function ( url, file, retryCount, s, f ) {
                new FileTransfer().download( encodeURI( url ), file, function() {
                    numberOfFilesDownloaded = numberOfFilesDownloaded + 1;
                    showPercentage(numberOfFilesDownloaded * 100 / filesArray.length);
                    s();
                }, function() {
                    if (retryCount < app.download.numRetries) {
                        setTimeout(function() {
                            downloadFile(url, file, retryCount + 1, s, f);
                        }, 100);
                    } else {
                        f();
                    }
                } );
            };
            var showPercentage = function (percent) {
                if (game) {
                    $('.' + game + ' .progress .message').html('loading');
                    $('.' + game + ' .progress .percent').html(percent.toFixed(0) + '%');
                    $('.' + game + ' .progress').show();
                } else {
                    $('#preapp').html('Loading ' + percent.toFixed(0) + '%');
                }
            };
            var functions = [];
            for ( var i = 0; i < filesArray.length; i++ ) {
                var file = filesArray[ i ];
                // Add the retryCount parameter
                file.push(0);
                functions.push( downloadFile );
                functions.push( file );
            }
            app.util.executeAll(functions, 3, successFunction, failureFunction);
        },
        getGameDetails: function(game, doneFunction) {
            app.download.getJSON( app.siteDomain + '/game/' + game.code + '/details.json', function (detailsdata) {
                detailsdata.filesURL.replace(new RegExp('\\/' + game.code + '\\/v_[0-9]*'), '/' + game.code + 'v_' + game.version);
                app.download.getJSON( app.gamesDomain + detailsdata.filesURL, function(filesdata) {
                    var gamesdetails = {};
                    gamesdetails.files = filesdata.files;
                    gamesdetails.total = filesdata.total;
                    gamesdetails.name = detailsdata.name;
                    gamesdetails.code = detailsdata.code;
                    gamesdetails.version = detailsdata.version;
                    gamesdetails.orientation = detailsdata.orientation;
                    app.util.writeObjectAsJSON(gamesdetails, 'content/' + game.code, 'details.json', doneFunction, doneFunction);
                }, app.util.error)
            }, app.util.error)
        },
        checkAndDownload: function ( game, download, doneFunction ) {
            var callback = function(details) {
                app.download.downloadFromDetailsJSON(details, game, download, doneFunction);
            };
            $.ajax('/content/' + game.code + '/details.json').done(callback).fail(function() {
                app.download.getGameDetails( game , callback);
            });
        },
        downloadFromDetailsJSON: function(details, game, download, doneFunction) {
            if (typeof details === 'string') {
                details = JSON.parse(details);
            }
            var files = JSON.parse( JSON.stringify( details.files ).replace( /"\//g, '"/v_' + game.version + '/' ) );
            files.push( '/iconsole.js' );
            files.push( '/PM.js' );
            files.push( '/commsWrapper.js' );
            app.download.checkAndDownloadFiles( details, files, doneFunction, download );
        },
        checkAndDownloadFiles: function ( obj, filesArray, completeFunction, download ) {
            var game = obj.code;
            var index = 0;
            var downloadFunction = function(file, successFunction, failureFunction ) {
                $('.' + game + ' .progress').show();
                $('.' + game + ' .progress .message').html('loading');
                app.download.downloadGameFileIfNeeded( game, file, download, 0, function() {
                    index = index + 1;
                    app.ui.setPercent('.' + game + ' .progress .percent', 100 * (index ) / filesArray.length);
                    successFunction();
                }, failureFunction );
            };
            var failureFunction = function ( error ) {
                if ( error ) {
                    console.log( error );
                }
                obj.failed = true;
                completeFunction();
            };
            var downloads = [];
            for (var i = 0; i < filesArray.length; i++) {
                downloads.push(downloadFunction);
                downloads.push( [ filesArray[ i ] ] );
            }
            app.util.executeAll( downloads, 6, completeFunction, failureFunction );
        },
        downloadGameFileIfNeeded: function ( game, file, download, retryCount, successFunction, failureFunction ) {
            var retryableFailureFunction = function() {
                if (retryCount < app.download.numRetries) {
                    setTimeout(function() {
                        app.download.downloadGameFileIfNeeded( game, file, download, retryCount + 1, successFunction, failureFunction);
                    }, 100);
                } else {
                    failureFunction();
                }
            };
            resolveLocalFileSystemURL( cordova.file.dataDirectory + '/content/' + game + file, successFunction,
                function () {
                    if ( download ) {
                        if ( file === '/iconsole.js' ) {
                            app.util.copyFile( 'iconsole.js', cordova.file.dataDirectory + 'app/js/console/',
                                cordova.file.dataDirectory + '/content/' + game + '/',
                                successFunction, retryableFailureFunction );
                        } else if ( file === '/PM.js') {
                            app.util.copyFile( 'PM.js', cordova.file.dataDirectory + 'app/js/console/',
                                cordova.file.dataDirectory + '/content/' + game + '/',
                                successFunction, retryableFailureFunction );
                        } else if ( file === '/commsWrapper.js') {
                            app.util.copyFile( 'commsWrapper.js', cordova.file.dataDirectory + 'app/js/console/',
                                cordova.file.dataDirectory + '/content/' + game + '/',
                                successFunction, retryableFailureFunction );
                        } else {
                            new FileTransfer().download( encodeURI( app.gamesDomain + '/m/iwin/' + game + file ),
                                cordova.file.dataDirectory + '/content/' + game + file,
                                successFunction, retryableFailureFunction );
                        }
                    } else {
                        retryableFailureFunction();
                    }
                } );
        },
        downloadFileIfNeeded: function ( sourceURL, destinationFile, retryCount, successFunction, failureFunction ) {
            resolveLocalFileSystemURL( destinationFile, successFunction,
                function () {
                    new FileTransfer().download( encodeURI( sourceURL ), destinationFile, successFunction, function() {
                        if (retryCount < app.download.numRetries) {
                            setTimeout(function() {
                                app.download.downloadFileIfNeeded( sourceURL, destinationFile, retryCount + 1, successFunction, failureFunction );
                            }, 100);
                        } else {
                            failureFunction();
                        }
                    } );
                } );
        },
        getJSON: function( uri, success, failure, keep) {
            var file = app.download.tempfileBase + '-' + app.download.tempfile++ + '.json';
            new FileTransfer().download( encodeURI(uri), cordova.file.dataDirectory + 'tmp/' + file, function() {
                $.ajax('/tmp/' + file).done( function(data) {
                    if (typeof data === 'string') {
                        data = JSON.parse(data);
                    }
                    if (keep) {
                        success(data, '/tmp/' + file);
                    } else {
                    resolveLocalFileSystemURL( cordova.file.dataDirectory + 'tmp/' + file, function(fileToRemove) {
                            fileToRemove.remove(function() {
                                success(data);
                            }, failure);
                        }, failure);
                    }
                })
            }, failure);
        },
        checkCurrentSiteVersion: function() {
            app.download.getJSON(app.siteDomain + '/site-version.json', app.download.downloadIfNewSiteVersion, app.util.error, false);
            return false;
        },
        downloadIfNewSiteVersion: function(data) {
            data = (typeof data === 'string') ? JSON.parse(data) : data;
            if (data['site-version'] > app.siteVersion) {
                app.download.getJSON(app.siteDomain + '/gamesapp/site.json', app.download.downloadSiteJSONImages, app.util.error, false);
            }
        },
        downloadSiteJSONImages: function(data) {
            data = (typeof data === 'string') ? JSON.parse(data) : data;
            if (typeof data.advertConfiguration === 'undefined') {
                data.advertConfiguration = app.ads.advertConfiguration;
            }
            var oldSitePrefix = app.appState.sitePrefix;
            var newSitePrefix = app.util.swapSitePrefix();
            var id = 0;
            var deletions = [];
            var downloads = [];
            var input = [data.banner, data.games];
            for (var i = 0; i < input.length; i++) {
                var type = input[i];
                for (var j = 0; j < type.length; j++) {
                    var image = type[j].icon;
                    var ext = /(?:\.([^.]+))?$/.exec(image)[1];
                    var newImage = '/app/site/' + newSitePrefix + '/' + id + (ext ? '.' + ext : '');
                    id = id + 1;
                    type[j].icon = newImage;
                    deletions.push(app.util.deleteFile);
                    deletions.push([cordova.file.dataDirectory + newImage]);
                    downloads.push(app.download.downloadFileIfNeeded);
                    downloads.push([image, cordova.file.dataDirectory + newImage, 0]);
                }
            }
            app.util.executeAll(deletions, 3, function() {
                app.util.executeAll(downloads, 3, function() {
                    app.download.saveSiteJSON(data, newSitePrefix, oldSitePrefix);
                }, app.util.error);
            }, app.util.error);
        },
        saveSiteJSON: function(data, newSitePrefix, oldSitePrefix) {
            app.util.writeObjectAsJSON(data, 'app/site/' + newSitePrefix, 'site.json', function() {
                app.onBackFunction = function() {
                    app.onBackFunction = null;
                    var chain = [
                        app.util.deleteFile,
                        [cordova.file.dataDirectory + 'app/site.json'],
                        app.util.moveFile,
                        [cordova.file.dataDirectory + 'app/site/' + newSitePrefix + '/site.json', 'app']
                    ];
                    if (oldSitePrefix) {
                        chain.push(app.util.deleteFolder);
                        chain.push([cordova.file.dataDirectory + 'app/site/' + oldSitePrefix]);
                    }
                    app.util.chainFunction(chain, function(){
                        app.util.location.reload();
                    }, app.util.error);
                };
                // If we're not playing a game right now, reload the content
                if  (!app.util.isGameCurrentlyPlaying()) {
                    app.onBackFunction();
                }
            }, app.util.error);
        }
    };
})();