(function() {
    window.handlers = {
        initialise: function() {
            CommsWrapper.addCommandHandler( 'gameReady', handlers.game.ready );
            CommsWrapper.addCommandHandler( 'gameLoaded', handlers.game.loaded );
            CommsWrapper.addCommandHandler( 'getGameData', handlers.game.getGameData );
            CommsWrapper.addCommandHandler( 'setGameData', handlers.game.setGameData );
            CommsWrapper.addCommandHandler( 'postHighScore', handlers.game.postHighScore );
            CommsWrapper.addCommandHandler( 'gameLoadProgress', handlers.game.progress );
            CommsWrapper.addCommandHandler( 'requestLocale', handlers.game.requestLocale );
            CommsWrapper.addCommandHandler( 'showAdvert', handlers.ads.show );
            CommsWrapper.addCommandHandler( 'periodicalGetData', handlers.periodical.getData );
            CommsWrapper.addCommandHandler( 'periodicalGetPeriods', handlers.periodical.getPeriods );
            CommsWrapper.addCommandHandler( 'console', handlers.consoleWrite );
        },
        currentGame: '',
        periodicalData: {
            url: '',
            mappings: {},
            periods: []
        },
        game: {
            readyCallback: null,
            ready: function ( request ) {
                app.ui.showGameFrame();
                handlers.game.readyCallback = request;
            },
            loaded: function ( request ) {
                request.result();
                var callback = handlers.game.readyCallback;
                handlers.game.readyCallback = null;
                callback.result( { user: handlers.user, console: { type: 'app' } } );
            },
            getGameData: function ( request ) {
                request.result( JSON.stringify( handlers.user.data ) );
            },
            setGameData: function ( request ) {
                handlers.user.data = request.data;
                app.util.saveIndividualGameProgress(handlers.currentGame, handlers.user, app.util.emptyFunction);
                request.result( {} );
            },
            postHighScore: function ( request ) {
                request.result( {} );
            },
            progress: function ( request ) {
                $('.innerloader').css('width', '' + request.data.progress * 100 + '%');
                request.result( {} );
            },
            requestLocale: function ( request ) {
                var locales = request.data.locales || [];
                var currentLocale = 'en-us-dummy';
                var chosenLocale = null;
                if (locales.length === 0) {
                    chosenLocale = '';
                }
                var lowerLocales = [];
                for (var i = 0; i < locales.length; i++) {
                    lowerLocales.push(locales[i].toLowerCase());
                }
                // Recursive function
                var getSuitableLocale = function(searchedLocale, lowerLocales, locales) {
                    var matches = /(.*)-.*?/.exec(searchedLocale);
                    if (matches) {
                        searchedLocale = matches[1];
                        var regex = new RegExp(searchedLocale + '.*');
                        for (var i = 0; i < lowerLocales.length; i++) {
                            if (regex.test(lowerLocales[i])) {
                                return locales[i];
                            }
                        }
                        /*
                         If we've come this far, we haven't found a matching locale, so
                         search one locale up
                         */
                        return getSuitableLocale(searchedLocale, lowerLocales, locales);
                    } else {
                        return null;
                    }
                };
                /*
                 Test to see if there are any locales that are provided which offer a larger
                 language support. So take the currentLocale, and strip off the last _.*, and
                 see if any of the locales provided match... i.e. for each locale provided,
                 see if it matches STRIPPED.* (this will cater for locales like en, as well as
                 en_US where
                 */
                if (chosenLocale === null) {
                    chosenLocale = getSuitableLocale(currentLocale, lowerLocales, locales);
                }
                /*
                 If we've come this far and haven't selected a locale, then we fall back to
                 the first locale on the given list;
                 */
                if (chosenLocale === null) {
                    chosenLocale = locales[0];
                }
                request.result({locale: chosenLocale});
            }
        },
        ads: {
            show: function ( request ) {
                app.ads.incrementAdvertAttemptAndConditionallyShowAdvert(function() {
                    request.result();
                });
            }
        },
        user: {},
        periodical: {
            getData: function( request ) {
                var requestData = request.data;
                if ( requestData.periodsBack > handlers.periodicalData.periods.length ) {
                    requestData.periodsBack = handlers.periodicalData.periods.length - 1;
                }
                if ( requestData.periodsBack < 0 ) {
                    requestData.periodsBack = 0;
                }
                
                while (
                Object.keys(handlers.periodicalData.mappings).length > 0 // Only do this if we are using mappings
                    &&
                requestData.periodsBack < handlers.periodicalData.periods.length
                    &&
                        handlers.periodicalData.mappings[handlers.periodicalData.periods[requestData.periodsBack].json] === undefined
                    ) {
                    requestData.periodsBack++;
                }
                if (requestData.periodsBack === handlers.periodicalData.periods.length) {
                    request.result( { data: null, period: -1});
                } else {
                    var periodical = Object.keys(handlers.periodicalData.mappings).length > 0 ?
                        handlers.periodicalData.mappings[handlers.periodicalData.periods[requestData.periodsBack].json] :
                        handlers.periodicalData.url + handlers.periodicalData.periods[requestData.periodsBack].json; // Only use the mappings if they exist
                    $.ajax(periodical).done(function(data){
                        if ( typeof data === 'string' ) {
                            data = JSON.parse(data);
                        }
                        request.result( { data: data, period: requestData.periodsBack } );
                    }).fail(function(){
                        request.result( { data: null, period: -1});
                    });
                }
            },
            getPeriods: function ( request ) {
                request.result( handlers.periodicalData.periods );
            }
        },
        resetUser: function() {
            handlers.user = {
                name: '',
                avatar: '',
                level: 0,
                progress: 0,
                highScore: 0,
                xpNeeded: 0,
                data: {}
            };
        },
        consoleWrite: function( request ) {
            request.data.args.splice(0, 0, 'Game Frame: ')
            console[request.data.type].apply(console[request.data.type], request.data.args);
            request.result({});
        },
        mute: function() {
            return CommsWrapper.postMessage('event', {type: 'soundOff'});
        },
        unMute: function() {
            return CommsWrapper.postMessage('event', {type: 'soundOn'});
        }
    };
})();