(function() {
    /* istanbul ignore next */
    window.app = window.app || {};
    $.extend( window.app, {
        fullUI: ((app.util.location.protocol === 'http:' || app.util.location.protocol === 'https:' ) && app.util.location.host !== 'localhost:12344' && app.util.location.href !== 'http://localhost:8080/index.html') || navigator.userAgent.match(/PhantomJS/) !== null,
        initialised: false,
        gameStates: {},
        siteDomain: '',
        gamesDomain: '',
        prebundle: {},
        appState: {
            sitePrefix: ''
        },
        backHasBeenClicked: undefined,
        isPaused: true,
        overrideGameVersions: false,
        // Application Constructor
        initialize: function () {
            this.bindEvents();
            this.util.loadAppStateFromLS();
            this.util.loadGameStateFromLS();
            handlers.resetUser();
        },
        // Bind Event Listeners
        //
        // Bind any events that are required on startup. Common events are:
        // 'load', 'deviceready', 'offline', and 'online'.
        bindEvents: function () {
            app.isPaused = false;
            document.addEventListener( 'deviceready', this.onDeviceReady, false );
            document.addEventListener( 'DOMContentLoaded', this.onDocumentReady, false );
            document.addEventListener( 'backButton', this.backClicked, false );
            document.addEventListener( 'pause', this.pauseToBackground, false);
            document.addEventListener( 'resume' , this.resumedFromBackground, false );
            document.addEventListener( 'online', app.ui.updateNetworkIndicatorIconAtTop, false);
            document.addEventListener( 'offline', app.ui.updateNetworkIndicatorIconAtTop, false);
            handlers.initialise();
        },
        switchToFullUI: function() {
            if (!app.fullUI) {
                app.util.location.href = 'http://localhost:8080/app/index.html';
            }
        },
        // deviceready Event Handler
        //
        // The scope of 'this' is the event. In order to call the 'receivedEvent'
        // function, we must explicitly call 'app.receivedEvent(...);'
        onDeviceReady: function () {
            StatusBar.hide();
            if ( ( app.util.location.protocol === 'file:' && !app.util.containsHash( 'serverup' ) ) || app.util.location.host === 'localhost:12344' ) {
                app.util.startInitialServerAndRedirectForFullUI();
            } else {
                if (!app.fullUI) {
                    app.util.copyAppFiles(app.switchToFullUI);
                } else {
                    app.util.displayFullUI();
                }
            }
        },
        onDocumentReady: function () {
            if ( !app.initialised ) {
                //navigator.splashscreen.hide();
                if (app.fullUI) {
                    $( '#boardelement a').on( 'click', app.ui.checkIfOfflineAndDisplayWarningOrPlayGame );
                    $( '#bannerelement a' ).on( 'click', app.ui.checkIfOfflineAndDisplayWarningOrPlayGame );
                    $( '#clearcache a' ).on( 'click', app.clearCacheClicked ).show();
                    $( '#fps a' ).on( 'click', app.ui.showFPS );
                    $( '#consoleshow a' ).on( 'click', app.showConsole );
                    $( '#infront a' ).on( 'click', app.backClicked );
                    $('#mainapp').css('display','inherit');
                    $('#preapp').css('display','none');
                    $('a.settings').on( 'click', app.ui.showSettingsModal).show();
                    $('.showsettings').on( 'click', app.showSettingsMenu);
                    $('#settingsmodal').on('hide.bs.modal', app.ui.modalHidden);
                    $('#offlinemodal').on('hide.bs.modal', app.ui.modalHidden);
                    $('#console').on('click', app.toggleConsoleExpand);
                    $('#checknewcontent').on( 'click', app.download.checkCurrentSiteVersion);
                    $('.networkindicator').on( 'click', app.ui.checkIfOfflineAndDisplayWarningOrPlayGame);
                    $('#overridegameversions input').on( 'click', app.overrideGameVersionsClicked );
                    $('#addgame a').on( 'click', app.util.addCustomGame );
                    $('#setsitedomain a').on('click', app.util.setSiteDomain );
                }
                $( window ).bind( 'orientationchange', app.ui.rotateDevice );
                $( window ).resize( app.ui.rotateDevice );
                $( window ).trigger( 'orientationchange', true );
                app.consoleDiv = document.getElementById('console');
                app.overrideConsole();
                app.initialised = true;
                app.ui.carouselSlideForTablet();
            }
        },
        resumedFromBackground: function ( event ) {
            app.isPaused = false;
            if (cordova.platformId === 'ios') {
                var start = function () {
                    app.util.startServer(function () {
                    }, app.util.error);
                };
                cordova.plugins.CorHttpd.stopServer(start, start);
            }
            handlers.unMute();
        },
        pauseToBackground: function( event ) {
            app.isPaused = true;
            handlers.mute();
        },
        playGameClicked: function ( event, dontShowAnim ) {
            var target = $( event.target );
            while ( target.length > 0 ) {
                var game = target.data( 'game' ); 
                if ( game ) {
                    var playGame = function() {
                        app.playGameOnceClickAnimationCompleted(target, event);
                    };
                    if (dontShowAnim) {
                        playGame();
                    } else {
                        app.ui.clickAnim('img.' + game, playGame);
                    }
                    return false;
                } else {
                    target = target.parent();
                }
            }
        },
        playGameOnceClickAnimationCompleted: function(target, event) {
            if (target.hasClass('prebundle')) {
                return app.prebundledGameClicked(event);
            } else {
                var game = target.data('game');
                if (app.gameStates[game].version) {
                    app.deleteAllOtherVersionsIfOnlineAndDifferentVersion(game, target, event);
                } else {
                    app.util.findHighestGameVersionOnFile(game, function() {
                        app.deleteAllOtherVersionsIfOnlineAndDifferentVersion(game, target, event);
                    }, target.data('version'))
                }
            }
        },
        deleteAllOtherVersionsIfOnlineAndDifferentVersion: function(game, target, event) {
            if (app.util.online()) {
                app.util.findAllVersionsOfGameOnFile(game, function (versions) {
                    var version = target.data('version');
                    versions = versions.filter(function (value) {
                        return value !== version;
                    });
                    var chain = [];
                    for (var i = 0; i < versions.length; i++) {
                        chain.push(app.util.deleteFolder);
                        chain.push([cordova.file.dataDirectory + '/content/' + game + '/v_' + versions[i]]);
                    }
                    if (versions.length > 0) {
                        var gameState = app.gameStates[game];
                        gameState.downloaded = false;
                        gameState.version = version;
                        chain.push(app.util.deleteFile);
                        chain.push([cordova.file.dataDirectory + '/content/' + game + '/details.json']);
                    }
                    chain.push(app.playGameOnceGameVersionObtained);
                    chain.push([target, event]);
                    app.util.chainFunction(chain);
                });
            } else {
                app.playGameOnceGameVersionObtained(target, event);
            }
        },
        playGameOnceGameVersionObtained: function(target, event) {
            var chain = [];
            var game = target.data('game');
            var gameState = app.gameStates[game];
            var version = gameState.version;
            if (app.overrideGameVersions) {
                var chosenVersion = prompt('Please override what version of ' + game + ' you want to use', version);
                if (version !== chosenVersion) {
                    app.util.deleteFolder(cordova.file.dataDirectory + '/content/' + game + '/v_' + version);
                    target.data('version', chosenVersion);
                    version = chosenVersion;
                    gameState.downloaded = false;
                    gameState.version = version;
                    chain.push(app.util.deleteFile);
                    chain.push([cordova.file.dataDirectory + '/content/' + game + '/details.json']);
                }
            }
            if (gameState.downloaded) {
                app.gameIsDownloadedNowPlayGame(game, version);
            } else if (!gameState.downloading) {
                app.util.chainFunction(chain, function(){
                    app.downloadThenPlayGame(game, event, version);
                }, app.util.error);
            }
        },
        gameIsDownloadedNowPlayGame: function(game, version) {
            // Modify this when we have per game network indicator
            if ($('.' + game + '.onlinerequired').length === 0) {
                $('.'+ game + ' .networkindicator').remove();
            }
            app.util.chainFunction([
                app.util.deleteFile, [cordova.file.dataDirectory + '/content/' + game + '/iconsole.js'],
                app.util.deleteFile, [cordova.file.dataDirectory + '/content/' + game + '/PM.js'],
                app.util.deleteFile, [cordova.file.dataDirectory + '/content/' + game + '/commsWrapper.js'],
                app.util.copyFile, ['iconsole.js', cordova.file.dataDirectory + '/app/js/console/', cordova.file.dataDirectory + '/content/' + game + '/'],
                app.util.copyFile, ['PM.js', cordova.file.dataDirectory + '/app/js/console/', cordova.file.dataDirectory + '/content/' + game + '/'],
                app.util.copyFile, ['commsWrapper.js', cordova.file.dataDirectory + '/app/js/console/', cordova.file.dataDirectory + '/content/' + game + '/'],
                app.util.storeOrientationFromDetails,[game]
            ], function () {
                app.util.loadIndividualGameProgress(game, function () {
                    handlers.currentGame = game;
                    app.playGame(game, version);
                });
            }, /* istanbul ignore next */ function () {
            });
        },
        downloadThenPlayGame: function(game, event, version) {
            var gameState = app.gameStates[game];
            gameState.downloading = true;
            app.download.checkAndDownload({code: game, version: version}, true, function () {
                gameState.downloaded = true;
                gameState.downloading = false;
                gameState.version = version;
                app.util.storeGameStateToLS();
                $('.' + game + ' .progress').hide();
                app.ensureGameStateObjectExists(game);
                app.playGameClicked(event, true);
            });
        },
        prebundledGameClicked: function(event) {
            var target = $( event.target );
            while ( target.length > 0 ) {
                if (target.data('game')) {
                    var game = target.data('game');
                    app.copyPreBundledGames(game, event);
                    return false;
                }
                target = target.parent();
            }
        },
        clearCacheClicked: function () {
            app.clearCache();
            return false;
        },
        playGame: function ( game, version ) {
            $.ajax( app.gameSource( game, version ) + 'game_details.json').done(function(game_details) {
                if (typeof game_details === 'string') {
                    game_details = JSON.parse(game_details);
                }
                var backButtonLocation = 'tl';
                if ( game_details && game_details.html5app ) {
                    backButtonLocation = game_details.html5app.backbutton;
                } 
                app.ui.hideAppUI( backButtonLocation );
                app.handlePeriodicalPriorToLaunchingGame(game_details, game, version);
            });
        },
        gameSource: function ( game, gameVersion ) {
            return '/content/' + game + '/v_' + gameVersion + '/';
        },
        clearCache: function () {
            resolveLocalFileSystemURL( cordova.file.dataDirectory + 'app', function ( appdir ) {
                appdir.removeRecursively( function () {
                    resolveLocalFileSystemURL(cordova.file.dataDirectory + '/content', function ( file ) {
                        app.gameStates = {};
                        app.util.storeGameStateToLS();
                        file.removeRecursively( function () {
                            var callback = function() {
                                window.plugins.webViewReloader.reload();
                            };
                            cordova.plugins.CorHttpd.stopServer(callback, callback);
                        });
                    });
                } );
            }, function () {
                console.log( arguments );
            } );
        },
        isAndroid: (/(android)/i.test( navigator.userAgent )),
        ensureGameStatesExistsAndRemoveOnlineRequiredFlagsForDownloadedGames: function() {
            var games = [];
            // The following gets a list of all the games on the board by using the class name of the divs, but has to
            // strip the onlinerequired class if it exists so that the list is just the games codes.
            $.each( $('#board > div'), function(){
                games.push(this.className.replace(/ onlinerequired/,''));
            });
            games=$.unique(games);
            for (var i = 0; i < games.length; i++) {
                var game = games[i];
                app.ensureGameStateObjectExists(game);
                app.removeOnlineRequiredFlagIfNecessary(game);
            }
        },
        ensureGameStateObjectExists: function ( game ) {
            if ( !app.gameStates[ game ] ) {
                app.gameStates[ game ] = {};
                app.util.storeGameStateToLS();
            }
        },
        removeOnlineRequiredFlagIfNecessary: function( game ) {
            var networkindicator = $( '.' + game ).not('.onlinerequired').find('.networkindicator');
            if ( app.gameStates[ game].downloaded && networkindicator.length > 0 ) {
                networkindicator.remove();
            }
        },
        backClicked: function() {
            if (app.util.isGameCurrentlyPlaying()) {
                app.ui.clickAnim('img.backbutton',
                    app.backHasBeenClicked === undefined ? app.closeGame : app.util.emptyFunction
                );
            } else if (app.modalShown) {
                app.modalShown.modal('hide');
            } else {
                window.plugins.Suspend.suspendApp();
            }
        },
        closeGame: function() {
            app.backHasBeenClicked = true;
            console.log('Back button clicked');
            app.flurryAnalytics.endTimedEvent('gameplayed', { game: handlers.currentGame }, /* istanbul ignore next */ function() { console.log('recorded game finished');}, /* istanbul ignore next */ function() { console.error('couldn\'t record game finished');});
            app.ui.animateClosingGame();
        },
        debugClicked: 0,
        showSettingsMenu: function() {
            if(++app.debugClicked >= 5) {
                app.ui.showSettingsModal();
            }
            return false;
        },
        overrideConsole: function() {
            var oldConsole = {
                debug: console.debug,
                error: console.error,
                log: console.log,
                info: console.info,
                warn: console.warn
            };
            console.debug = function() { app.addConsoleLine('debug', arguments, oldConsole); };
            console.error = function() { app.addConsoleLine('error', arguments, oldConsole); };
            console.log = function() { app.addConsoleLine('log', arguments, oldConsole); };
            console.info = function() { app.addConsoleLine('info', arguments, oldConsole); };
            console.warn = function() { app.addConsoleLine('warn', arguments, oldConsole); };
        },
        consoleDiv: null,
        addConsoleLine: function(type, args, oldConsole) {
            if (app.consoleDiv) {
                var span = app.consoleDiv.firstChild;
                span.className = type;
                span.innerHTML = JSON.stringify(Array.prototype.slice.apply(args));
                oldConsole[type].apply(console, args);
                app.consoleDiv.appendChild(span);
            }
        },
        showConsole: function() {
            $('#console').show();
            app.expandedConsole = true;
            app.toggleConsoleExpand();
        },
        expandedConsole: false,
        toggleConsoleExpand: function() {
            if (app.expandedConsole) {
                $('#console').animate({left:window.innerWidth - 40 + 'px', right: '-' + (window.innerWidth - 40) + 'px', height: '48px'}, 500);
                app.expandedConsole = false;
            } else {
                $('#console').animate({left: '0px', right: '0px', height: '100%'}, 500);
                app.expandedConsole = true;
            }
        },
        copyPreBundledGames: function(gameToLaunch, event) {
            $.ajax('/app/prebundle_allfiles.json').done(function (data) {
                if (typeof(data) === 'string') {
                    data = JSON.parse(data);
                    var lastGame = '';
                    var chainPre = [];
                    var chain = [];
                    var files = [];
                    var source = cordova.platformId === 'ios' ? 'http://localhost:12344' : cordova.file.applicationDirectory + 'www';
                    for (var i = 0; i < data.allFiles.length +1; i++) {
                        if (i === data.allFiles.length || lastGame !== '' && data.allFiles[i].lastIndexOf('/' + lastGame ) === -1) {
                            var chainToPushGameTo = (lastGame === gameToLaunch) ? chainPre : chain;
                            chainToPushGameTo.push(function(game, doneFunction, failureFunction){
                                app.gameStates[ game ].downloading = true;
                                app.gameStates[ game ].downloaded = false;
                                $('.' + game + ' .progress .message').html('loading');
                                $('.' + game + ' .progress .percent').html('0%');
                                $('.' + game + ' .progress').show();
                                doneFunction();
                            });
                            chainToPushGameTo.push([lastGame]);
                            chainToPushGameTo.push(app.download.downloadLocalFiles);
                            chainToPushGameTo.push([files, lastGame]);
                            files = [];
                            chainToPushGameTo.push(function(game, doneFunction, failureFunction){
                                app.gameStates[ game ].downloading = false;
                                app.gameStates[ game ].downloaded = true;
                                $('.' + game + ' .progress').hide();
                                $('.' + game + ' a').removeClass('prebundle');
                                doneFunction();
                            });
                            chainToPushGameTo.push([lastGame]);
                            // Set the downloaded flag to the game to true
                            if (i !== data.allFiles.length) {
                                lastGame = data.allFiles[i].substring(1, data.allFiles[i].indexOf('/', 1));
                            }
                            if (chainToPushGameTo === chainPre) {
                                chainToPushGameTo.push(function(game, doneFunction) {
                                    app.playGameClicked(event, true);
                                    doneFunction();
                                });
                                chainToPushGameTo.push([gameToLaunch]);
                            }
                        } else {
                            if (lastGame === '') {
                                lastGame = data.allFiles[i].substring(1, data.allFiles[i].indexOf('/', 1));
                            }
                        }
                        if (i !== data.allFiles.length) {
                            files.push([source + '/prebundle' + data.allFiles[i], cordova.file.dataDirectory + 'content' + data.allFiles[i]]);
                        }
                    }
                    Array.prototype.unshift.apply(chain, chainPre);
                    chain.push(app.util.deleteFile);
                    chain.push([cordova.file.dataDirectory + 'app/prebundle.json']);
                    app.util.chainFunction(chain, function(){console.log(arguments);}, function(){console.log(arguments);});
                }
            });
        },
        handlePeriodicalPriorToLaunchingGame: function(game_details, game, version) {
            if (game_details.periodical) {
                app.download.getJSON( app.gamesDomain + '/m/iwin/' + game + '/periodical/periodical.json', function(periodical) {
                    if (typeof periodical === 'object') {
                        periodical = JSON.stringify(periodical);
                    }
                    periodical = JSON.parse(periodical);
                    handlers.periodicalData = {
                        url: 'http://localhost:8080/onlinegames/m/iwin/' + game + '/periodical/',
                        mappings: {},
                        periods: periodical.periods
                    };
                    app.launchGame(game, version, game_details);
                }, app.util.emptyFunction, true);
            } else {
                handlers.periodicalData = {
                    url: '',
                    mappings: {},
                    periods: []
                };
                app.launchGame(game, version, game_details);
            }
        },
        launchGame: function(game, version, game_details) {
            if (!app.util.isGameCurrentlyPlaying()) {
                var preGame = game_details && game_details.advertBehavior && (game_details.advertBehavior.preGame || typeof game_details.advertBehavior.preGame === 'undefined');
                app.ads.clearAdvertAttemptNumber(preGame);
                var iFrame = CommsWrapper.loadIFrame( app.gameSource( game, version ) );
                iFrame.scrolling = 'no';
                iFrame.onload = app.ui.rotateDevice;
                iFrame.style.display = 'inherit';
                CommsWrapper.postMessage( 'init', {
                    width: window.innerWidth,
                    height: window.innerHeight,
                    iap: { supported: false, provider: 'gamesapp', analyticsSupported: false },
                    periodical: handlers.periodicalData
                } );
                var gameObject = window[ game ];
                if ( gameObject && gameObject.orientation ) {
                    screen.lockOrientation( gameObject.orientation );
                } else {
                    screen.unlockOrientation();
                }
                app.flurryAnalytics.startTimedEvent('gameplayed', { game: handlers.currentGame }, function() { console.log('recorded game started');}, function() { console.error('couldn\'t record game started');});
            }
        },
        overrideGameVersionsClicked: function() {
            app.overrideGameVersions = $('#overridegameversions input').is(':checked');
        }
    });
    app.initialize();
    
})();