/**
 * @author mrdoob / http://mrdoob.com/
 *
 * Massively edited for performance and reduction of file size
 */

var Stats = function () {
    var timer = window.performance ? window.performance : Date;
    var startTime = timer.now(), prevTime = startTime;
    var ms = 0;
    var fps = 0;
    var frames = 0, mode = false;
    var width = 'width:', height = 'height:', backgroundColor = 'background-color:';
    var textCss = 'font-family:sans-serif;font-size:9px;font-weight:bold;line-' + height +'15px;color:#0f';
    var padding = 'padding:0 0 3px 3px;text-align:left;' + backgroundColor +'#0';
    var position = 'position:relative;' + width + '94px;' + height + '30px;' + backgroundColor + '#0f';
    var div = 'div', span='span', block = 'block', none = 'none';

    function decorate(type, id, cssText, parent) {
        var element = document.createElement(type);
        if (id){
            element.id = id;
        }
        element.style.cssText = cssText;
        if (parent) {
            parent.appendChild(element);
        }
        return element;
    }
    function createGraph(parent, bgColour){
        while(parent.children.length < 94) {
            decorate(span, null, width +'1px;' + height + '30px;float:left;' + backgroundColor + bgColour, parent);
        }
    }
    function updateGraph( dom, value, max ) {
        dom.appendChild( dom.firstChild ).style.height = (30 * Math.min(1, 1 - (value / max))).toFixed(0) + 'px';
    }

    function setMode( value ) {
        mode = value;
        fpsDivStyle.display = mode ? none : block;
        msDivStyle.display = mode ? block : none;
    }

    var container = decorate(div, 'stats', width +'100px;opacity:0.9');
    container.addEventListener( 'mousedown', function ( event ) { event.preventDefault(); setMode( !mode ) }, false );

    var fpsDiv = decorate(div, 'fps', padding + '02', container);
    var fpsDivStyle = fpsDiv.style;

    var fpsText = decorate(div, 'fpsText', textCss + 'f;', fpsDiv);
    fpsText.innerHTML = 'FPS';

    var fpsGraph = decorate(div, 'fpsGraph', position + 'f', fpsDiv);
    createGraph(fpsGraph, '#113');

    var msDiv = decorate(div, 'ms', padding + '20;', container);
    var msDivStyle = msDiv.style;

    var msText = decorate(div, 'msText', textCss + '0;', msDiv);
    msText.innerHTML = 'MS';

    var msGraph = decorate(div, 'msGraph', position + '0', msDiv);
    createGraph(msGraph, '#131');

    return {

        domElement: container,

        setMode: setMode,

        begin: function () {

            startTime = timer.now();

        },

        end: function () {

            var time = timer.now();

            ms = time - startTime;

            msText.textContent = ms.toFixed(2) + ' MS';
            updateGraph( msGraph, ms , 200 );

            frames ++;

            if ( time > prevTime + 1000 ) {

                fps = ( frames * 1000 ) / ( time - prevTime );

                fpsText.textContent = fps.toFixed(1) + ' FPS';
                updateGraph(fpsGraph, fps, 100);

                prevTime = time;
                frames = 0;

            }
        }
    }

};