(function(){
    /* istanbul ignore next */
    window.app = window.app || {};
    window.app.ui = {
        setPercent: function(selector, percentage) {
            if (percentage < 0) { percentage = 0;}
            if (percentage > 100) { percentage = 100;}

            $(selector).html(percentage.toFixed(0) + '%');
        },
        hideAppUI: function ( backButtonPlacement ) {
            if (typeof backButtonPlacement === 'undefined') {
                backButtonPlacement = 'tl';
            }
            $('.main').addClass('fadehide');
            $('#infront').addClass(backButtonPlacement);
        },
        showGameFrame: function() {
            $('#infront').addClass('fadeshow');
            $('.topbar').addClass('fadehide');
            $( '#mainframe' ).addClass('fadeshow');
            $( '#background' ).addClass('fadehide');
        },
        carouselSlideForTablet: function () {
            $(".carousel-control.left").bind("touchstart", function(event){
                $('.carousel').carousel('prev');
            });

            $(".carousel-control.right").bind("touchstart", function(event){
                $('.carousel').carousel('next');
            });

            $(".carousel-control").bind("touchend", function(event){
                $('.carousel').carousel('cycle');
            });
        },
        clickAnim: function(target, callback) {
            var now = Date.now();
            $(target).addClass('clickanim');
            var animin = function() {
                cancelAnimationFrame(animin);
                var newNow = Date.now();
                if (newNow - now < 166) {
                    requestAnimationFrame(animin);
                } else {
                    now = newNow;
                    $(target).removeClass('clickanim');
                    var animout = function() { 
                        cancelAnimationFrame(animout); 
                        if (Date.now() - now < 166) {
                            requestAnimationFrame(animout); 
                        } else {
                            callback();
                        } 
                    }; 
                    animout(); 
                } 
            };
            animin();
        },
        addGameToBanner: function ( game, active, prebundled ) {
            var code = game.code;
            var version = game.version;
            var img = game.icon;
            var onlineRequired = game.onlineRequired;
            var template = $( '#bannerelement div.cloneme' );
            var added = template.clone( true );
            added.removeClass( 'cloneme' );
            added.addClass( code );
            if (active) {
                added.addClass( 'active' );
            }
            if (prebundled) {
                added.find( 'a' ).addClass( 'prebundle' );
                if (!onlineRequired) {
                    added.find('.networkindicator').remove();
                }
            }
            if (onlineRequired) {
                added.addClass( 'onlinerequired' );
            }
            added.find( 'a' ).data( 'game', code ).data( 'version', version );
            added.find( 'img.billboard' ).attr( 'src', img );
            added.find( 'img.billboard' ).addClass( code );
            return added[0]; // De-jQuery this
        },
        addGameToBoard: function ( game, prebundled ) {
            var code = game.code;
            var version = game.version;
            var img = game.icon;
            var onlineRequired = game.onlineRequired;
            var template = $( '#boardelement div.cloneme' );
            var added = template.clone( true );
            added.removeClass( 'cloneme' );
            added.addClass( code );
            if (prebundled) {
                added.find( 'a' ).addClass( 'prebundle' );
                if (!onlineRequired) {
                    added.find('.networkindicator').remove();
                }
            }
            if (onlineRequired) {
                added.addClass( 'onlinerequired' );
            }
            added.find( 'a' ).data( 'game', code ).data( 'version', version );
            added.find( 'img.icon' ).attr( 'src', img );
            added.find( 'img.icon' ).addClass( code );
            return added[0]; // De-jQuery this
        },
        rotateDevice: function () {
            document.getElementsByTagName( 'body' )[ 0 ].className = window.innerWidth > window.innerHeight ? 'landscape' : /* istanbul ignore next */ 'portrait';
            $('#mainapp').height(window.innerHeight - $('.topbar').height());
            $('#background').height(window.innerHeight).width(window.innerWidth);
        },
        animateClosingGame: function() {
            $('#infront').removeClass();
            $('#background').removeClass('fadehide');
            $('#mainframe').removeClass('fadeshow');
            setTimeout(function () {
                app.backHasBeenClicked = undefined;
                $('#mainframe').attr('src', '/app/blank.html');
                $( '.main' ).removeClass('fadehide');
                $( '.topbar' ).removeClass('fadehide');
                screen.unlockOrientation();
                if (app.onBackFunction) {
                    app.onBackFunction();
                }
            }, 500);
        },
        fps: false,
        /* Ignoring the showFPS function, as it's taken from online code */
        showFPS:  /* istanbul ignore next */ function() {
            if (!app.ui.fps) {
                var stats = new Stats();
                stats.setMode(false); // false: fps, true: ms
                var style = stats.domElement.style;
                // align top-left
                style.position = 'absolute';
                style.left = style.bottom = '0px';
                style.zIndex = '99999';

                var raf = window.requestAnimationFrame;
                var callbackFunctions = [];
                window.requestAnimationFrame = function (callback) {
                    callbackFunctions.push(callback);
                };
                var update = function () {

                    stats.begin();

                    var localCallBackFunctions = callbackFunctions;
                    callbackFunctions = [];
                    for (var i = 0; i < localCallBackFunctions.length; i++) {
                        localCallBackFunctions[i]();
                    }

                    stats.end();

                    raf(update);

                };

                update();

                var bodyInterval = setInterval(function () {
                    if (document.body) {
                        clearInterval(bodyInterval);
                        document.body.appendChild(stats.domElement);
                    }
                }, 100);
                app.ui.fps = true;
            }
        },
        updateNetworkIndicatorIconAtTop: function() {
            if (!app.isPaused && !app.util.isGameCurrentlyPlaying()) {
                var type = navigator.connection.type;
                var networkIndicatorIcon = $('.networkindicator').not('#mainapp .networkindicator').not('#template .networkindicator').not('#offlinemodal .networkindicator');
                var classList = networkIndicatorIcon.attr('class');
                if (classList) {
                    var oldClass = '';
                    if (classList.match(/none/)) {
                        oldClass = 'none';
                    }
                    var newClass = '';
                    switch (type) {
                        case 'ethernet':
                        case 'wifi':
                        case '2g':
                        case '3g':
                        case '4g':
                        case 'cell':
                        case 'cellular':
                            break;
                        case 'none':
                            newClass = 'none';
                            break;
                    }
                    if (newClass !== oldClass) {
                        var raf = requestAnimationFrame(function() {
                            cancelAnimationFrame(raf);
                            if (oldClass !== '') {
                                networkIndicatorIcon.removeClass(oldClass);
                            }
                            if (newClass !== '') {
                                networkIndicatorIcon.addClass(newClass);
                            }
                            app.ui.updateNetworkIndicatorIconOnGames(oldClass, newClass);
                        });
                    }
                }
            }
        },
        updateNetworkIndicatorIconOnGames: function(oldClass, newClass) {
            var networkIndicatorIcon = $('#mainapp .networkindicator').not('#offlinemodal .networkindicator');
            if (oldClass !== '') {
                networkIndicatorIcon.removeClass(oldClass);
            }
            if (newClass === 'none') {
                networkIndicatorIcon.addClass(newClass);
            }
        },
        checkIfOfflineAndDisplayWarningOrPlayGame: function(event) {
            var target = $( event.target );
            while ( target.length > 0 ) {
                var game = target.data('game');
                if (game) {
                    target = $(target).parent().find('.networkindicator');
                    if (target.hasClass('none')) {
                        app.modalShown = $('#offlinemodal');
                        app.modalShown.modal('show');
                    } else {
                        app.playGameClicked(event, false);
                    }
                    return false;
                } else {
                    target = target.parent();
                }
            }
        },
        showSettingsModal: function() {
            app.modalShown = $('#settingsmodal');
            app.modalShown.modal('show');
        },
        modalHidden: function() {
            app.modalShown = null;
        },
        setPrebundleFlagsOnGames: function(prebundle) {
            if (typeof prebundle === 'string') {
                prebundle = JSON.parse(prebundle);
            }
            app.prebundle = prebundle;
            app.util.loadGameStateFromLS();
            for (var game in prebundle) {
                // Don't prepare any disabled games
                if (game.lastIndexOf('_', 0) === -1) {
                    app.gameStates[game] = app.gameStates[game] || {};
                }
            }
            app.util.storeGameStateToLS();
            app.ui.getSiteJsonAndUpdateUI(prebundle);
        },
        getSiteJsonAndUpdateUI: function(prebundle) {
            $.ajax('site.json').done(function (siteJson) {
                if (typeof siteJson === 'string') {
                    siteJson = JSON.parse(siteJson);
                }
                app.siteDomain = siteJson.siteDomain;
                app.gamesDomain = siteJson.gamesDomain;
                app.siteVersion = siteJson['site-version'];
                app.ads.advertConfiguration = siteJson.advertConfiguration;
                // Bounce the local server, now we have the correct games domain.
                app.util.chainFunction([
                    cordova.plugins.CorHttpd.stopServer,
                    app.util.startServer,
                    app.ui.displayGamesIcons, [siteJson, prebundle]
                ], app.util.emptyFunction, app.util.error);
            });
        },
        displayGamesIcons: function(siteJson, prebundle) {
            var game;
            var games = siteJson.banner;
            var bannerGames = [];
            var boardGames = [];

            // Pre calculate what we're going to add to the banner and board
            for (var i = 0; i < games.length; i++) {
                game = games[i];
                bannerGames.push(app.ui.addGameToBanner(game, i === 0, prebundle && typeof prebundle[game.code] !== 'undefined'));
            }
            games = siteJson.games;
            for (var i = 0; i < games.length; i++) {
                game = games[i];
                boardGames.push(app.ui.addGameToBoard(game, prebundle && typeof prebundle[game.code] !== 'undefined'));
            }
            var banner = $( '#banner' )[0]; // De-jQuery this
            var board = $( '#board' )[0]; // De-jQuery this
            // Make the changes to the banner and board inside an Animation Frame for UI speed.
            var raf = requestAnimationFrame(function() {
                cancelAnimationFrame(raf);
                for (var i = 0; i < bannerGames.length; i++) {
                    banner.appendChild(bannerGames[i]);
                }
                for (var i = 0; i < boardGames.length; i++) {
                    board.appendChild(boardGames[i]);
                }
                app.ensureGameStatesExistsAndRemoveOnlineRequiredFlagsForDownloadedGames();
            });
            app.download.checkCurrentSiteVersion();
        }
    };
})();